@@ -7,7 +7,7 @@ from PIL import Image
 from janim.components.component import Component
 
 
-class Cmpt_Image(Component):
+class Cmpt_Image[ItemT](Component[ItemT]):
     '''
     图像组件，包含一个 PIL 图像以及 ``min_mag_filter``
     '''
@@ -25,10 +25,10 @@ class Cmpt_Image(Component):
         self.set(other.img, other.min_mag_filter)
         return self
 
-    def __eq__(self, other: Cmpt_Image) -> bool:
+    def not_changed(self, other: Cmpt_Image) -> bool:
         return id(self.img) == id(other.img) and self.min_mag_filter == other.min_mag_filter
 
-    def set(self, img: Image.Image | None, min_mag_filter: int | None) -> Self:
+    def set(self, img: Image.Image | None = None, min_mag_filter: int | None = None) -> Self:
         '''
         设置 PIL 图像
         '''
