@@ -6,35 +6,42 @@ from typing import Callable, Iterable, Self
 import numpy as np
 
 import janim.utils.refresh as refresh
+from janim.anims.method_updater_meta import register_updater
 from janim.components.component import Component
 from janim.constants import (DEFAULT_ITEM_TO_EDGE_BUFF,
                              DEFAULT_ITEM_TO_ITEM_BUFF, DOWN, IN, LEFT,
                              MED_SMALL_BUFF, ORIGIN, OUT, PI, RIGHT, UP)
 from janim.exception import InvaildMatrixError, PointError
 from janim.items.item import Item
+from janim.locale.i18n import get_local_strings
 from janim.typing import Vect, VectArray
 from janim.utils.bezier import integer_interpolate, interpolate
 from janim.utils.config import Config
-from janim.utils.data import AlignedData
+from janim.utils.data import AlignedData, Array
 from janim.utils.iterables import resize_and_repeatedly_extend
 from janim.utils.paths import PathFunc, straight_path
 from janim.utils.signal import Signal
-from janim.utils.space_ops import angle_of_vector, get_norm, rotation_matrix
-from janim.utils.unique_nparray import UniqueNparray
+from janim.utils.space_ops import (angle_of_vector, get_norm, normalize,
+                                   rotation_between_vectors, rotation_matrix)
+
+_ = get_local_strings('points')
 
 type PointsFn = Callable[[np.ndarray], VectArray]
 type PointFn = Callable[[np.ndarray], Vect]
 type ComplexFn = Callable[[complex], complex]
 
+DEFAULT_POINTS_ARRAY = Array()
+DEFAULT_POINTS_ARRAY.data = np.zeros((0, 3))
+
 
 class Cmpt_Points[ItemT](Component[ItemT]):
     resize_func = staticmethod(resize_and_repeatedly_extend)
+    ''''''
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self._points = UniqueNparray()
-        self.clear()
+        self._points = DEFAULT_POINTS_ARRAY.copy()
 
     def init_bind(self, bind: Component.BindInfo):
         super().init_bind(bind)
@@ -49,11 +56,12 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         return cmpt_copy
 
     def become(self, other: Cmpt_Points) -> Self:
-        self._points = other._points.copy()
-        Cmpt_Points.set.emit(self)
+        if not self._points.is_share(other._points):
+            self._points = other._points.copy()
+            Cmpt_Points.set.emit(self)
         return self
 
-    def __eq__(self, other: Cmpt_Points) -> bool:
+    def not_changed(self, other: Cmpt_Points) -> bool:
         return self._points.is_share(other._points)
 
     @classmethod
@@ -78,10 +86,8 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         *,
         path_func: PathFunc = straight_path
     ) -> None:
-        if cmpt1 == cmpt2:
-            return
-
-        self.set(path_func(cmpt1.get(), cmpt2.get(), alpha))
+        if not cmpt1._points.is_share(cmpt2._points):
+            self.set(path_func(cmpt1.get(), cmpt2.get(), alpha))
 
     # region 点数据 | Points
 
@@ -95,15 +101,10 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         '''
         得到自己以及后代物件的所有点坐标数据
         '''
-        point_datas = [self.get()]
-
-        if self.bind is not None:
-            for item in self.bind.at_item.walk_descendants(self.bind.decl_cls):
-                cmpt = self.get_same_cmpt_without_mock(item)
-                if cmpt is None:
-                    continue
-                point_datas.append(cmpt.get())
-
+        point_datas = [
+            cmpt.get()
+            for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock()
+        ]
         return np.vstack(point_datas)
 
     @Signal
@@ -113,8 +114,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
 
         使用形如 ``.set([[1.5, 3, 2], [2, 1.5, 0]])`` 的形式
         '''
-        if not isinstance(points, np.ndarray):
-            points = np.array(points)
+        points = np.asarray(points)
         if points.size == 0:
             points = np.zeros((0, 3))
 
@@ -133,7 +133,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
 
     def clear(self) -> Self:
         '''清除点'''
-        self.set(np.zeros((0, 3)))
+        self.set(DEFAULT_POINTS_ARRAY.data)
         return self
 
     def extend(self, points: VectArray) -> Self:
@@ -176,14 +176,14 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         得到 ``points`` 的第一个点
         '''
         self._raise_error_if_no_points()
-        return self._points._data[0].copy()
+        return self._points.data[0]
 
     def get_end(self) -> np.ndarray:
         '''
         得到 ``points`` 的最后一个点
         '''
         self._raise_error_if_no_points()
-        return self._points._data[-1].copy()
+        return self._points.data[-1]
 
     def get_start_and_end(self) -> tuple[np.ndarray, np.ndarray]:
         '''
@@ -192,19 +192,18 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         return (self.get_start(), self.get_end())
 
     def point_from_proportion(self, alpha: float) -> np.ndarray:
-        points = self._points._data
+        points = self._points.data
         i, subalpha = integer_interpolate(0, len(points) - 1, alpha)
         return interpolate(points[i], points[i + 1], subalpha)
 
     def pfp(self, alpha) -> np.ndarray:
-        '''point_from_proportion 的缩写'''
+        '''``point_from_proportion`` 的缩写'''
         return self.point_from_proportion(alpha)
 
     def _raise_error_if_no_points(self) -> None:
         if not self.has():
             name = inspect.currentframe().f_back.f_code.co_name
-            # TODO: i18n
-            raise PointError(f'Cannot call {name} with no points')
+            raise PointError(_('Cannot call {name} with no points').format(name=name))
 
     # endregion
 
@@ -217,23 +216,15 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         '''
         表示物件（包括后代物件）的矩形包围框
         '''
-        box_datas = []
-
-        if self.has():
-            box_datas.append(self.self_box.data)
-
-        if self.bind is not None:
-            for item in self.bind.at_item.walk_descendants(self.bind.decl_cls):
-                cmpt = self.get_same_cmpt_without_mock(item)
-                if cmpt is None or not cmpt.has():
-                    continue
-
-                box_datas.append(cmpt.self_box.data)
-
+        box_datas = [
+            cmpt.self_box.data
+            for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock()
+            if cmpt.has()
+        ]
         return self.BoundingBox(np.vstack(box_datas) if box_datas else [])
 
     @property
-    @set.self_refresh()
+    @set.self_refresh
     @refresh.register
     def self_box(self) -> BoundingBox:
         '''
@@ -253,13 +244,13 @@ class Cmpt_Points[ItemT](Component[ItemT]):
             '''
             根据传入的 ``points`` 计算得到包围框的 左下、中心、右上 三个点
             '''
-            points = np.array(points)
+            points = np.asarray(points)
 
             if len(points) == 0:
                 return np.zeros((3, 3))
 
-            mins = np.nanmin(points, 0)
-            maxs = np.nanmax(points, 0)
+            mins = np.nanmin(points, axis=0)
+            maxs = np.nanmax(points, axis=0)
             mids = (mins + maxs) / 2
 
             return np.array([mins, mids, maxs])
@@ -298,14 +289,22 @@ class Cmpt_Points[ItemT](Component[ItemT]):
                 )
             )
 
-        def get_corners(self) -> list[np.ndarray]:
+        def get_corners(self) -> np.ndarray:
             '''得到包围框（立方体）的八个顶点'''
-            return [
-                self.get(x + y + z)
-                for x in (LEFT, RIGHT)
-                for y in (DOWN, UP)
-                for z in (IN, OUT)
-            ]
+            x1, y1, z1 = self.data[0]
+            x2, y2, z2 = self.data[2]
+
+            # 不直接使用 `self.get` 是因为它太慢了
+            return np.array([
+                [x1, y1, z1],
+                [x1, y1, z2],
+                [x1, y2, z1],
+                [x1, y2, z2],
+                [x2, y1, z1],
+                [x2, y1, z2],
+                [x2, y2, z1],
+                [x2, y2, z2],
+            ])
 
         @property
         def top(self) -> np.ndarray:
@@ -385,7 +384,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         *,
         about_point: Vect | None = None,
         about_edge: Vect | None = ORIGIN,
-        root_only: bool = False,
+        root_only: bool = False
     ) -> Self:
         '''
         将所有点作为单独的一个参数传入 ``func``，并将 ``func`` 返回的结果作为新的点坐标数据
@@ -398,7 +397,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
             else:
                 about_point = self.box.get(about_edge)
 
-        def apply(cmpt: Cmpt_Points):
+        for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock(root_only):
             if cmpt.has():
                 if about_point is None:
                     cmpt.set(func(cmpt.get()))
@@ -407,16 +406,6 @@ class Cmpt_Points[ItemT](Component[ItemT]):
 
             Cmpt_Points.apply_points_fn.emit(cmpt, func, about_point)
 
-        apply(self)
-
-        if not root_only and self.bind is not None:
-            for item in self.bind.at_item.walk_descendants(self.bind.decl_cls):
-                cmpt = self.get_same_cmpt_without_mock(item)
-                if cmpt is None:
-                    continue
-
-                apply(cmpt)
-
         return self
 
     def apply_point_fn(
@@ -452,8 +441,8 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         matrix = np.array(matrix)
         if matrix.shape not in ((2, 2), (3, 3)):
             raise InvaildMatrixError(
-                '只有 2x2 或 3x3 矩阵是有效的，'
-                f'而传入的是 {"x".join(str(v) for v in matrix.shape)} 矩阵'
+                _('Only 2x2 or 3x3 matrix are valid, but a {shape} matrix was passed in')
+                .format(shape="x".join(str(v) for v in matrix.shape))
             )
 
         if about_point is None and about_edge is None:
@@ -464,7 +453,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         full_matrix[:matrix.shape[0], :matrix.shape[1]] = matrix
 
         self.apply_points_fn(
-            lambda points: np.dot(points, full_matrix.T),
+            lambda points: points @ full_matrix.T,
             about_point=about_point,
             about_edge=about_edge,
             root_only=root_only
@@ -499,6 +488,11 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         )
         return self
 
+    @register_updater(
+        lambda self, p, angle, **kwargs:
+            self.rotate(angle * p.alpha, **kwargs),
+        grouply=True
+    )
     def rotate(
         self,
         angle: float,
@@ -513,7 +507,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         '''
         rot_matrix_T = rotation_matrix(angle, axis).T
         self.apply_points_fn(
-            lambda points: np.dot(points, rot_matrix_T),
+            lambda points: points @ rot_matrix_T,
             about_point=about_point,
             about_edge=about_edge,
             root_only=root_only
@@ -540,6 +534,11 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         )
         return self
 
+    @register_updater(
+        lambda self, p, scale_factor, **kwargs:
+            self.scale((np.asarray(scale_factor) - 1) * p.alpha + 1, **kwargs),
+        grouply=True
+    )
     def scale(
         self,
         scale_factor: float | Iterable,
@@ -556,9 +555,13 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         例如传入 ``scale_factor`` 为 ``(2, 0.5, 1)`` 则是在 ``x`` 方向上缩放为两倍，在 ``y`` 方向上压缩为原来的一半，在 ``z`` 方向上保持不变
         '''
         if isinstance(scale_factor, Iterable):
-            scale_factor = np.array(scale_factor).clip(min=min_scale_factor)
+            sgn = np.sign(scale_factor)
+            scale_factor = sgn * abs(np.asarray(scale_factor)).clip(min=min_scale_factor)
         else:
-            scale_factor = max(scale_factor, min_scale_factor)
+            if scale_factor >= 0:
+                scale_factor = max(scale_factor, min_scale_factor)
+            else:
+                scale_factor = min(scale_factor, -min_scale_factor)
 
         self.apply_points_fn(
             lambda points: scale_factor * points,
@@ -568,6 +571,11 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         )
         return self
 
+    @register_updater(
+        lambda self, p, factor, **kwargs:
+            self.stretch((factor - 1) * p.alpha + 1, **kwargs),
+        grouply=True
+    )
     def stretch(
         self,
         factor: float,
@@ -740,6 +748,42 @@ class Cmpt_Points[ItemT](Component[ItemT]):
 
         return self
 
+    @register_updater(
+        lambda self, p, factor=0.2, direction=RIGHT, **kwargs:
+            self.shear(factor * p.alpha, direction, **kwargs),
+        grouply=True
+    )
+    def shear(
+        self,
+        factor: float = 0.2,
+        direction: Vect = RIGHT,
+        *,
+        about_point: Vect | None = None,
+        about_edge: Vect = ORIGIN,
+        root_only: bool = False
+    ) -> Self:
+        '''
+        切变
+
+        - ``factor`` 表示切变的程度
+        - ``direction`` 表示切变的方向
+        - 可以传入 ``about_point`` 或 ``about_edge`` 控制参考点
+        '''
+        mat_shear = [
+            [1, factor, 0],
+            [0, 1, 0],
+            [0, 0, 1]
+        ]
+        if np.isclose(normalize(direction), RIGHT).all():
+            mat = mat_shear
+        else:
+            mat_rot = rotation_between_vectors(direction, RIGHT)
+            # mat_rot.T == mat_rot.I
+            mat = mat_rot.T @ mat_shear @ mat_rot
+
+        self.apply_matrix(mat, about_point=about_point, about_edge=about_edge, root_only=root_only)
+        return self
+
     def put_start_and_end_on(self, start: Vect, end: Vect) -> Self:
         '''
         通过旋转和缩放，使得物件的起点和终点被置于 ``start`` 和 ``end``
@@ -747,7 +791,7 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         curr_start, curr_end = self.get_start(), self.get_end()
         curr_vect = curr_end - curr_start
         if np.all(curr_vect == 0):
-            raise PointError("Cannot position endpoints of closed loop")
+            raise PointError(_('Cannot position endpoints of closed loop'))
         target_vect = end - start
         self.scale(
             get_norm(target_vect) / get_norm(curr_vect),
@@ -767,6 +811,10 @@ class Cmpt_Points[ItemT](Component[ItemT]):
 
     # region 位移 | movement
 
+    @register_updater(
+        lambda self, p, vector, *, root_only=False:
+            self.shift(np.asarray(vector) * p.alpha, root_only=root_only)
+    )
     def shift(self, vector: Vect, *, root_only=False) -> Self:
         '''
         相对移动 ``vector`` 向量
@@ -778,6 +826,23 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         )
         return self
 
+    @staticmethod
+    def _compute_move_shift(
+        src: Cmpt_Points,
+        target: Item | Vect,
+        aligned_edge: Vect,
+        coor_mask: Iterable,
+        root_only: bool = False,
+        item_root_only: bool = False
+    ) -> np.ndarray:
+        if isinstance(target, Item):
+            cmpt = src.get_same_cmpt(target)
+            box = cmpt.self_box if item_root_only else cmpt.box
+            target = box.get(aligned_edge)
+
+        point_to_align = (src.self_box if root_only else src.box).get(aligned_edge)
+        return (target - point_to_align) * coor_mask
+
     def move_to(
         self,
         target: Item | Vect,
@@ -790,20 +855,54 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         '''
         移动到 ``target`` 的位置
         '''
-        if isinstance(target, Item):
-            cmpt = self.get_same_cmpt(target)
-            box = cmpt.self_box if item_root_only else cmpt.box
-            target = box.get(aligned_edge)
+        self.shift(
+            self._compute_move_shift(self, target, aligned_edge, coor_mask, root_only, item_root_only),
+            root_only=root_only
+        )
+        return self
 
-        point_to_align = self.box.get(aligned_edge)
-        self.shift((target - point_to_align) * coor_mask, root_only=root_only)
+    def move_to_by_indicator(
+        self,
+        indicator: Item,
+        target: Item,
+        *,
+        aligned_edge: Vect = ORIGIN,
+        coor_mask: Iterable = (1, 1, 1),
+        root_only: bool = False,
+        indicator_root_only: bool = False,
+        item_root_only: bool = False
+    ) -> Self:
+        '''
+        与 :meth:`move_to` 类似，但是该方法作用 ``indicator`` 被移动到 ``target`` 所计算出的位移，
+        而不是 :meth:`move_to` 中 ``self`` 被移动到 ``target`` 的位移
 
+        例如：
+
+        .. code-block:: python
+
+            t1 = TypstMath('x^2 + y^2')
+            t2 = TypstMath('x + y')
+            t2.points.move_to_by_indicator(t2[1], t1[2])
+
+        可以将 ``t2`` 移动至 ``t1`` 的位置，
+        并且使得 ``t2`` 的加号与 ``t1`` 的加号对齐
+
+        .. note::
+
+            这个示例使用 :meth:`~.TypstDoc.match_pattern` 会更简洁
+        '''
+        cmpt = self.get_same_cmpt(indicator)
+        self.shift(
+            cmpt._compute_move_shift(cmpt, target, aligned_edge, coor_mask, indicator_root_only, item_root_only),
+            root_only=root_only
+        )
         return self
 
     def align_to(
         self,
         item_or_point: Item | Vect,
         direction: Vect = ORIGIN,
+        *,
         root_only: bool = False,
         item_root_only: bool = False,
     ) -> Self:
@@ -931,7 +1030,32 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         self.to_center()
         return self
 
-    def to_center(self, root_only=False) -> Self:
+    def arrange_by_offset(
+        self,
+        offset: Vect,
+        *,
+        aligned_edge: Vect = ORIGIN,
+        center: bool = True
+    ) -> Self:
+        if self.bind is None or not self.bind.at_item.children:
+            return self
+
+        cmpts = [
+            self.get_same_cmpt(item)
+            for item in self.bind.at_item.children
+        ]
+        offset = np.array(offset)
+
+        for cmpt1, cmpt2 in zip(cmpts, cmpts[1:]):
+            delta = cmpt2.box.get(aligned_edge) - cmpt1.box.get(aligned_edge)
+            cmpt2.shift(offset - delta)
+
+        if center:
+            self.to_center()
+
+        return self
+
+    def to_center(self, *, root_only=False) -> Self:
         '''
         移动到原点 ``(0, 0, 0)``
         '''
@@ -953,6 +1077,27 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         self.shift(shift_val)
         return self
 
+    @staticmethod
+    def _compute_next_to_shift(
+        src: Cmpt_Points,
+        target: Item | Vect,
+        direction: Vect,
+        buff: float,
+        aligned_edge: Vect,
+        coor_mask: Iterable,
+        root_only: bool,
+        item_root_only: bool
+    ) -> np.ndarray:
+        if isinstance(target, Item):
+            cmpt = src.get_same_cmpt(target)
+            box = cmpt.self_box if item_root_only else cmpt.box
+            target = box.get(aligned_edge + direction)
+
+        direction = np.asarray(direction)
+
+        point_to_align = (src.self_box if root_only else src.box).get(aligned_edge - direction)
+        return (target - point_to_align + buff * direction) * coor_mask
+
     def next_to(
         self,
         target: Item | Vect,
@@ -967,14 +1112,51 @@ class Cmpt_Points[ItemT](Component[ItemT]):
         '''
         将该物件放到 ``target`` 旁边
         '''
-        if isinstance(target, Item):
-            cmpt = self.get_same_cmpt(target)
-            box = cmpt.self_box if item_root_only else cmpt.box
-            target = box.get(aligned_edge + direction)
-
-        point_to_align = self.box.get(aligned_edge - direction)
         self.shift(
-            (target - point_to_align + buff * direction) * coor_mask,
+            self._compute_next_to_shift(self, target,
+                                        direction, buff, aligned_edge, coor_mask,
+                                        root_only, item_root_only),
+            root_only=root_only
+        )
+        return self
+
+    def next_to_by_indicator(
+        self,
+        indicator: Item,
+        target: Item | Vect,
+        direction: Vect = RIGHT,
+        *,
+        buff: float = DEFAULT_ITEM_TO_ITEM_BUFF,
+        aligned_edge: Vect = ORIGIN,
+        coor_mask: Iterable = (1, 1, 1),
+        root_only: bool = False,
+        indicator_root_only: bool = False,
+        item_root_only: bool = False,
+    ) -> Self:
+        '''
+        与 :meth:`next_to` 类似，但是该方法作用 ``indicator`` 被放到 ``target`` 旁边所计算出的位移，
+        而不是 :meth:`move_to` 中 ``self`` 被放到 ``target`` 旁边的位移
+
+        例如：
+
+        .. code-block:: python
+
+            t1 = Typst('x^2 + y^2')
+            t2 = Typst('x + y z w')
+            t2.points.next_to_by_indicator(t2[1], t1[2], DOWN)
+
+        可以将 ``t1`` 对齐到 ``t2`` 的下方，
+        并且使得 ``t1`` 的加号在 ``t2`` 的加号的正下方
+
+        .. note::
+
+            这个示例也可以使用字符索引
+        '''
+        cmpt = self.get_same_cmpt(indicator)
+        self.shift(
+            self._compute_next_to_shift(cmpt, target,
+                                        direction, buff, aligned_edge, coor_mask,
+                                        indicator_root_only, item_root_only),
             root_only=root_only
         )
         return self
