@@ -1,24 +1,33 @@
 from __future__ import annotations
 
+import math
+from enum import Enum
 from typing import Callable, Generator, Iterable, Self
 
 import numpy as np
 
+from janim.utils.paths import PathFunc, straight_path
 import janim.utils.refresh as refresh
-from janim.components.points import Cmpt_Points
-from janim.constants import NAN_POINT, OUT, RIGHT
+from janim.components.points import Cmpt_Points, PointsFn
+from janim.constants import NAN_POINT, ORIGIN, OUT, RIGHT, UP, DEGREES
 from janim.exception import PointError
 from janim.items.item import Item
+from janim.locale.i18n import get_local_strings
 from janim.logger import log
-from janim.typing import VectArray
-from janim.utils.bezier import (PathBuilder, bezier, integer_interpolate,
-                                inverse_interpolate,
-                                partial_quadratic_bezier_points)
+from janim.typing import Vect, VectArray
+from janim.utils.bezier import (PathBuilder,
+                                approx_smooth_quadratic_bezier_handles, bezier,
+                                integer_interpolate, inverse_interpolate,
+                                partial_quadratic_bezier_points,
+                                smooth_quadratic_path)
 from janim.utils.data import AlignedData
-from janim.utils.space_ops import get_norm, get_unit_normal
+from janim.utils.space_ops import (get_norm, get_unit_normal, normalize,
+                                   rotation_between_vectors)
+
+_ = get_local_strings('vpoints')
 
 
-class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
+class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT], impl=True):
     '''
     曲线点坐标数据
 
@@ -34,23 +43,41 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
 
       只有闭合的子路径，才能够进行填充色的渲染
     '''
-    def copy(self) -> Self:
-        return super().copy()
-
-    def become(self, other: Cmpt_VPoints) -> Self:
-        super().become(other)
-        return self
-
-    def __eq__(self, other: Cmpt_VPoints) -> bool:
-        return super().__eq__(other)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.make_smooth_after_applying_functions = False
 
     def set(self, points: VectArray) -> Self:
         if len(points) != 0 and len(points) % 2 == 0:
-            log.warning(f'设置的点数量为 {len(points)}，不是奇数，最后一个点被忽略')
+            log.warning(
+                _('The number of points set is {len}, which is not odd. The last point will be ignored.')
+                .format(len=len(points))
+            )
             points = points[:-1]
         super().set(points)
         return self
 
+    def apply_points_fn(
+        self,
+        func: PointsFn,
+        *,
+        about_point: Vect | None = None,
+        about_edge: Vect | None = ORIGIN,
+        root_only: bool = False
+    ) -> Self:
+        super().apply_points_fn(
+            func,
+            about_point=about_point,
+            about_edge=about_edge,
+            root_only=root_only
+        )
+        for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock(root_only):
+            if not isinstance(cmpt, Cmpt_VPoints) or not cmpt.make_smooth_after_applying_functions:
+                continue
+            cmpt.make_approximately_smooth()
+
+        return self
+
     # region align
 
     @classmethod
@@ -58,13 +85,13 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         cmpt1_copy = cmpt1.copy()
         cmpt2_copy = cmpt2.copy()
 
-        if cmpt1_copy == cmpt2_copy:
+        if cmpt1_copy.not_changed(cmpt2_copy):
             return AlignedData(cmpt1_copy, cmpt2_copy, cmpt1_copy.copy())
 
         if not cmpt1_copy.has():
-            cmpt1_copy.set([cmpt2.box.center])
+            cmpt1_copy.set([cmpt2.self_box.center])
         if not cmpt2_copy.has():
-            cmpt2_copy.set([cmpt1.box.center])
+            cmpt2_copy.set([cmpt1.self_box.center])
 
         subpaths1 = cmpt1_copy.get_subpaths()
         subpaths2 = cmpt2_copy.get_subpaths()
@@ -170,6 +197,20 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
 
         return AlignedData(cmpt1_copy, cmpt2_copy, cmpt1_copy.copy())
 
+    def interpolate(
+        self,
+        cmpt1: Self,
+        cmpt2: Self,
+        alpha: float,
+        *,
+        path_func: PathFunc = straight_path
+    ) -> None:
+        super().interpolate(cmpt1, cmpt2, alpha, path_func=path_func)
+        if np.all(cmpt1.get_closepath_flags() == cmpt2.get_closepath_flags()):
+            data = self.refresh_data[self.get_closepath_flags.__name__]
+            data.is_required = False
+            data.stored = cmpt1.get_closepath_flags()
+
     @staticmethod
     def align_path(path1: np.ndarray, path2: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
         diff = abs(len(path1) - len(path2)) // 2
@@ -210,6 +251,15 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
 
         return np.vstack(new_points)
 
+    def insert_n_curves(self, n: int, root_only=False) -> Self:
+        for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock(root_only):
+            if not isinstance(cmpt, Cmpt_VPoints) or cmpt.curves_count() == 0:
+                continue
+            points = cmpt.insert_n_curves_to_point_list(n, cmpt.get())
+            cmpt.set(points)
+
+        return self
+
     # endregion
 
     # region anchors and handles
@@ -226,16 +276,18 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         '''
         return self.get()[1::2]
 
-    def get_start_direction(self) -> np.ndarray:
-        points = self._points._data
+    @property
+    def start_direction(self) -> np.ndarray:
+        points = self._points.data
         start = points[0]
         for pos in points[1:]:
             if not np.isclose(start, pos).all():
                 return pos - start
         return RIGHT
 
-    def get_end_direction(self) -> np.ndarray:
-        points = self._points._data
+    @property
+    def end_direction(self) -> np.ndarray:
+        points = self._points.data
         end = points[-1]
         for pos in points[-2::-1]:
             if not np.isclose(end, pos).all():
@@ -249,7 +301,7 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         if len(indices) == 1:
             point = self.get_start()
         else:
-            point = self._points._data[indices[-2] + 2]
+            point = self._points.data[indices[-2] + 2]
         self.extend([(end + point) * 0.5, point])
         return self
 
@@ -280,8 +332,11 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         得到第 ``n`` 组的贝塞尔曲线控制点 (从 0 开始计数)
         '''
         if n < 0 or n >= self.curves_count():
-            raise PointError(f'n 必须是 0~{self.curves_count() - 1} 的值，{n} 无效')
-        return self._points._data[2 * n: 2 * n + 3].copy()
+            raise PointError(
+                _('n must be a value of 0~{maxn}, {n} is invalid')
+                .format(maxn=self.curves_count() - 1, n=n)
+            )
+        return self._points.data[2 * n: 2 * n + 3]
 
     def get_nth_curve_function(self, n: int) -> Callable[[float], np.ndarray]:
         '''
@@ -289,6 +344,21 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         '''
         return bezier(self.get_nth_curve_points(n))
 
+    def get_nth_curve_length_pieces(
+        self,
+        n: int,
+        sample_points: int | None = None,
+    ) -> np.ndarray:
+        if sample_points is None:
+            sample_points = 10
+
+        curve = self.get_nth_curve_function(n)
+        points = np.array([curve(a) for a in np.linspace(0, 1, sample_points)])
+        diffs = np.diff(points, axis=0)
+        norms = np.linalg.norm(diffs, axis=1)
+
+        return norms
+
     def quick_point_from_proportion(self, alpha: float) -> np.ndarray:
         '''
         相比 :meth:`point_from_proportion` 而言，更快
@@ -309,7 +379,7 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
             return (0, 0.0)
         partials: list[float] = [0]
         for tup in self.get_bezier_tuples():
-            if (tup[0] == tup[1]).all():
+            if np.isnan(tup[1]).all():
                 # Don't consider null curves
                 arclen = 0
             else:
@@ -318,7 +388,7 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
             partials.append(partials[-1] + arclen)
         full = partials[-1]
         if full == 0:
-            return len(partials), 1.0
+            return len(partials) - 2, 1.0
         # First index where the partial length is more than alpha times the full length
         index = next(
             (i for i, x in enumerate(partials) if x >= full * alpha),
@@ -338,16 +408,28 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         return self.get_nth_curve_function(index)(residue)
 
     def pointwise_become_partial(self, other: Cmpt_VPoints | Item, a: float, b: float) -> Self:
+        '''
+        将传入对象的曲线截取 ``[a, b]`` 区间（最大范围 ``[0, 1]`` 表示整个曲线）的部分后，设置到该对象上，且保持点的数量不变（将区间外的点都放到起点/终点处）
+        '''
         if isinstance(other, Item):
             cmpt = self.get_same_cmpt(other)
         else:
             cmpt = other
 
-        points = cmpt.get()
+        new_points = self.partial_points(cmpt.get(), a, b)
+        self.set(new_points)
+        return self
+
+    @staticmethod
+    def partial_points(points: np.ndarray, a: float, b: float) -> np.ndarray:
+        '''
+        得到 ``points`` 所表示的曲线中 ``[a, b]`` 的部分（最大范围 ``[0, 1]`` 表示整个曲线），且保持点的数量不变（将区间外的点都放到起点/终点处）
+
+        注：当 ``a <= 0`` 且 ``b >= 1`` 时，直接返回 ``points``，不作拷贝
+        '''
         if a <= 0 and b >= 1:
-            self.set(points)
-            return self
-        num_curves = cmpt.curves_count()
+            return points
+        num_curves = (len(points) - 1) // 2
 
         # Partial curve includes three portions:
         # - A start, which is some ending portion of an inner quadratic
@@ -364,7 +446,7 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         new_points = points.copy()
         if num_curves == 0:
             new_points[:] = 0
-            return self
+            return new_points
         if lower_index == upper_index:
             tup = partial_quadratic_bezier_points(points[i1:i2], lower_residue, upper_residue)
             new_points[:i1] = tup[0]
@@ -378,9 +460,50 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
             # Keep new_points i2:i3 as they are
             new_points[i3:i4] = high_tup
             new_points[i4:] = high_tup[2]
+
+        return new_points
+
+    def pointwise_become_partial_reduced(self, other: Cmpt_VPoints | Item, a: float, b: float) -> Self:
+        '''
+        将传入对象的曲线截取 ``[a, b]`` 区间（最大范围 ``[0, 1]`` 表示整个曲线）的部分后，设置到该对象上，丢弃区间外的点
+        '''
+        if isinstance(other, Item):
+            cmpt = self.get_same_cmpt(other)
+        else:
+            cmpt = other
+
+        new_points = self.partial_points_reduced(cmpt.get(), a, b)
         self.set(new_points)
         return self
 
+    @staticmethod
+    def partial_points_reduced(points: np.ndarray, a: float, b: float) -> np.ndarray:
+        '''
+        得到 ``points`` 所表示的曲线中 ``[a, b]`` 的部分（最大范围 ``[0, 1]`` 表示整个曲线），丢弃区间外的点
+
+        注：当 ``a <= 0`` 且 ``b >= 1`` 时，直接返回 ``points``，不作拷贝
+        '''
+        if a <= 0 and b >= 1:
+            return points
+        num_curves = (len(points) - 1) // 2
+        if num_curves == 0:
+            return np.zeros_like(points, dtype=points.dtype)
+
+        lower_index, lower_residue = integer_interpolate(0, num_curves, a)
+        upper_index, upper_residue = integer_interpolate(0, num_curves, b)
+        i1 = 2 * lower_index
+        i2 = 2 * lower_index + 3
+        i3 = 2 * upper_index
+        i4 = 2 * upper_index + 3
+
+        if lower_index == upper_index:
+            tup = partial_quadratic_bezier_points(points[i1:i2], lower_residue, upper_residue)
+            return np.array(tup, dtype=points.dtype)
+
+        low_tup = partial_quadratic_bezier_points(points[i1:i2], lower_residue, 1)
+        high_tup = partial_quadratic_bezier_points(points[i3:i4], 0, upper_residue)
+        return np.vstack([low_tup, points[i2 + 1: i3], high_tup[1:]])
+
     # endregion
 
     # region _as_corners 操作
@@ -407,10 +530,96 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         for point in points[1:]:
             builder.line_to(point)
         self.set(builder.get())
+        return self
 
     # endregion
 
-    # TODO: make_smooth
+    # region anchor mode
+
+    def is_smooth(self, angle_tol: float = 1 * DEGREES) -> bool:
+        return (self.get_joint_products() > math.cos(angle_tol)).all()
+
+    @Cmpt_Points.set.self_refresh
+    @refresh.register
+    def get_joint_products(self) -> np.ndarray:
+        '''
+        得到每个锚点前后方向向量的点积
+        '''
+        points = self.get()
+
+        vectors_count = self.curves_count() + 1
+        v1 = np.empty((vectors_count, 3), dtype=np.float32)
+        v2 = np.empty((vectors_count, 3), dtype=np.float32)
+
+        v1[1:] = points[2::2] - points[1::2]
+        v2[:-1] = points[1::2] - points[0:-1:2]
+
+        ends = np.array(self.get_subpath_end_indices())
+        starts = [0, *(ends[:-1] + 2)]
+        for start, end in zip(starts, ends):
+            if start == end:
+                continue
+            i1 = start // 2
+            i2 = end // 2
+            if np.isclose(points[start], points[end]).all():
+                v1[i1] = v1[i2]
+                v2[i2] = v2[i1]
+            else:
+                v1[i1] = v2[i1]
+                v2[i2] = v1[i2]
+
+        v1 /= np.sqrt(v1[:, 0]**2 + v1[:, 1]**2 + v1[:, 2]**2)[:, np.newaxis]
+        v2 /= np.sqrt(v2[:, 0]**2 + v2[:, 1]**2 + v2[:, 2]**2)[:, np.newaxis]
+        return np.sum(v1 * v2, axis=1)
+
+    def change_anchor_mode(self, mode: AnchorMode) -> Self:
+        if not self.has():
+            return self
+
+        subpaths = self.get_subpaths()
+        self.clear()
+        for subpath in subpaths:
+            new_subpath = subpath.copy()
+            anchors = subpath[::2]
+            match mode:
+                case AnchorMode.Jagged:
+                    new_subpath[1::2] = 0.5 * (anchors[:-1] + anchors[1:])
+                case AnchorMode.ApproxSmooth:
+                    new_subpath[1::2] = approx_smooth_quadratic_bezier_handles(anchors)
+                case AnchorMode.TrueSmooth:
+                    new_subpath = smooth_quadratic_path(anchors)
+            self.add_subpath(new_subpath)
+        return self
+
+    def make_smooth(self, approx=False, root_only=False) -> Self:
+        '''
+        Edits the path so as to pass smoothly through all
+        the current anchor points.
+
+        If approx is False, this may increase the total
+        number of points.
+        '''
+        mode = AnchorMode.ApproxSmooth if approx else AnchorMode.TrueSmooth
+        for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock(root_only):
+            if not isinstance(cmpt, Cmpt_VPoints):
+                continue
+            cmpt.change_anchor_mode(mode)
+
+        return self
+
+    def make_approximately_smooth(self, root_only=False) -> Self:
+        self.make_smooth(approx=True, root_only=root_only)
+        return self
+
+    def make_jagged(self, root_only=False) -> Self:
+        for cmpt in self.walk_same_cmpt_of_self_and_descendants_without_mock(root_only):
+            if not isinstance(cmpt, Cmpt_VPoints):
+                continue
+            cmpt.change_anchor_mode(AnchorMode.Jagged)
+
+        return self
+
+    # endregion
 
     # region unit_normal
 
@@ -432,7 +641,7 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         ])
 
     @property
-    @Cmpt_Points.set.self_refresh()
+    @Cmpt_Points.set.self_refresh
     @refresh.register
     def area_vector(self) -> np.ndarray:
         '''
@@ -441,7 +650,7 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         return self.get_area_vector_from_points(self.get())
 
     @property
-    @Cmpt_Points.set.self_refresh()
+    @Cmpt_Points.set.self_refresh
     @refresh.register
     def unit_normal(self) -> np.ndarray:
         if self.count() < 3:
@@ -474,6 +683,8 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
     def get_subpath_end_indices(self) -> list[int]:
         return list(self.walk_subpath_end_indices())
 
+    @Cmpt_Points.set.self_refresh
+    @refresh.register
     def get_closepath_flags(self) -> np.ndarray:
         '''
         得到子路径是否闭合的标志，结果长度与点数量相同
@@ -510,4 +721,85 @@ class Cmpt_VPoints[ItemT](Cmpt_Points[ItemT]):
         '''
         return self.get_parts_by_end_indices(self.get(), np.array(self.get_subpath_end_indices()))
 
+    def add_subpath(self, points: VectArray) -> Self:
+        if not self.has():
+            self.set(points)
+        else:
+            self.extend([NAN_POINT, *points])
+        return self
+
     # endregion
+
+    # region identity
+
+    @property
+    @Cmpt_Points.set.self_refresh
+    @refresh.register
+    def identity(self) -> tuple[int, np.ndarray]:
+        points = self.get()[:-1]
+        if len(points) < 2:
+            return RIGHT, 0
+
+        # 计算在初始方向上的宽度
+        vect = normalize(self.start_direction)
+        width = self.width_along_direction(vect)
+
+        # 将 points 旋转+归一化
+        rot = rotation_between_vectors(UP, vect)
+        points = points - points[0]
+        points @= rot   # points @ (UP->vect) == ((vect->UP) @ points.T).T
+        points /= width
+
+        # 把处于 round 分界部分的坐标稍微加一点
+        # 例如，两个相同的形状在归一化后可能同一个坐标被变换到 “比 0.45 稍微小一点的” 以及 “比 0.45 稍微大一点的”
+        # 它们分别会被 round 到 0.4 和 0.5，这是不符合预期的
+        # 所以这里加上 0.02 使得它们都会被 round 到 0.5，使得 hash 一致
+        points[np.abs(points % 0.1 - 0.05) < 0.01] += 0.02
+
+        np.round(points, 1, out=points)    # 原位 round
+        points[points == -0.0] = 0.0
+
+        # 计算结果：哈希值 以及 单位方向向量
+        vect.setflags(write=False)
+        return hash(points.tobytes()), vect
+
+    def width_along_direction(self, direction: Vect) -> float:
+        projections = np.dot(np.vstack(self.get_subpaths()), direction)
+        return np.max(projections) - np.min(projections)
+
+    def same_shape(self, other: Cmpt_VPoints | Item) -> bool:
+        '''
+        判断两组点是否有完全相同的形状
+
+        对于相同形状的两组点还可以用 :meth:`same_direction` 衡量方向重合度
+        '''
+        if isinstance(other, Item):
+            cmpt = self.get_same_cmpt(other)
+        else:
+            cmpt = other
+
+        return self.identity[0] == cmpt.identity[0]
+
+    def same_direction(self, other: Cmpt_VPoints | Item) -> bool:
+        '''
+        对于 :meth:`same_shape` 结果为 ``True`` 的两组点，可以通过该方法衡量方向重合度
+
+        - 返回 ``-1`` ~ ``1`` 之间的值
+        - 其中 ``1`` 表示完全同向，``-1`` 表示完全反向，``0`` 表示垂直
+
+        注：对于 :meth:`same_shape` 结果为 ``False`` 的两组点，该方法的结果没有实际含义
+        '''
+        if isinstance(other, Item):
+            cmpt = self.get_same_cmpt(other)
+        else:
+            cmpt = other
+
+        return np.dot(self.identity[1], cmpt.identity[1])
+
+    # endregion
+
+
+class AnchorMode(Enum):
+    Jagged = 0
+    ApproxSmooth = 1
+    TrueSmooth = 2
