@@ -64,3 +64,11 @@ RED = RED_C
 MAROON = MAROON_C
 PURPLE = PURPLE_C
 GREY = GREY_C
+
+# Animation labels
+C_LABEL_ANIM_DEFAULT = (192, 198, 205)
+C_LABEL_ANIM_IN = (107, 255, 185)
+C_LABEL_ANIM_STAY = (255, 255, 139)
+C_LABEL_ANIM_OUT = (98, 229, 255)
+C_LABEL_ANIM_ABSTRACT = (242, 218, 255)
+C_LABEL_ANIM_INDICATION = (255, 255, 127)
