@@ -1,7 +1,6 @@
 import sys
 from dataclasses import dataclass
 
-
 _sys_excepthook = sys.excepthook
 
 
@@ -16,14 +15,22 @@ sys.excepthook = custom_excepthook
 
 EXITCODE_PYSIDE6_NOT_FOUND = 1001
 '''``PySide6`` 未安装时的退出码'''
+EXITCODE_MODULE_NOT_FOUND = 1002
+'''使用 ``run`` 或 ``write`` 指定的文件未找到时的退出码'''
+EXITCODE_NOT_FILE = 1003
+'''使用 ``run`` 或 ``write`` 指定路径不是文件时的退出码'''
+EXITCODE_PYOPENGL_NOT_FOUND = 1004
+'''``PyOpenGL`` 未安装时的退出码（仅在设备不支持 OpenGL4.3 的情况下需要安装 PyOpenGL）'''
 
 EXITCODE_TYPST_NOT_FOUND = 1101
-'''``Typst`` 未安装时的退出码'''
+'''Typst 未安装时的退出码'''
 EXITCODE_TYPST_COMPILE_ERROR = 1102
-'''``Typst`` 编译失败时的退出码'''
+'''Typst 编译失败时的退出码'''
 
 EXITCODE_FFMPEG_NOT_FOUND = 2001
-'''``ffmpeg`` 未安装时的退出码'''
+'''ffmpeg 未安装时的退出码'''
+EXITCODE_FFPROBE_ERROR = 2002
+'''ffprobe 执行失败时的退出码'''
 
 
 class JAnimException(Exception): ...
@@ -40,10 +47,18 @@ class ExitException(JAnimException):
 
 class TimelineError(JAnimException): ...
 class TimelineLookupError(TimelineError): ...
-class StoreFailedError(TimelineError): ...
-class StoreNotFoundError(TimelineError): ...
+class NotAnimationError(TimelineError): ...
+class AnimationError(TimelineError): ...
+class AnimGroupError(AnimationError): ...
 
 
+class UpdaterError(JAnimException): ...
+
+
+class GetItemError(JAnimException): ...
+class PatternMismatchError(GetItemError): ...
+class InvalidOrdinalError(GetItemError): ...
+class InvalidTypstVarError(JAnimException): ...
 class CmptGroupLookupError(JAnimException): ...
 
 
