@@ -1,4 +1,5 @@
 
+from PySide6.QtCore import QSize
 from PySide6.QtGui import QResizeEvent
 from PySide6.QtWidgets import QWidget
 
@@ -7,16 +8,18 @@ class FixedRatioWidget(QWidget):
     '''
     使得传入的 ``inside`` 控件可以以固定比例塞在该控件中
     '''
-    def __init__(self, inside: QWidget, src_size: tuple[float, float], parent: QWidget | None = None) -> None:
+    def __init__(self, inside: QWidget, parent: QWidget | None = None) -> None:
         super().__init__(parent)
         self.inside = inside
         self.inside.setParent(self)
-        self.src_size = src_size
+        self.src_size = (1, 1)
 
-    def resizeEvent(self, event: QResizeEvent) -> None:
-        super().resizeEvent(event)
+    def set_src_size(self, size: tuple[float, float]) -> None:
+        self.src_size = size
+        self.update_inner_size(self.size())
 
-        wnd_width, wnd_height = event.size().toTuple()
+    def update_inner_size(self, wnd_size: QSize) -> None:
+        wnd_width, wnd_height = wnd_size.toTuple()
         w, h = get_proportional_scale_size(*self.src_size, wnd_width, wnd_height)
         self.inside.setGeometry(
             (wnd_width - w) // 2,
@@ -24,6 +27,10 @@ class FixedRatioWidget(QWidget):
             w, h
         )
 
+    def resizeEvent(self, event: QResizeEvent) -> None:
+        super().resizeEvent(event)
+        self.update_inner_size(event.size())
+
 
 def get_proportional_scale_size(src_width, src_height, tg_width, tg_height):
     '''
