@@ -1,9 +1,9 @@
+import types
 from typing import TYPE_CHECKING, Iterable, Protocol, runtime_checkable
 
-from janim.utils.data import AlignedData
-
 if TYPE_CHECKING:
     import numpy as np
+    from janim.anims.animation import Animation
 
 type Vect = Iterable[float] | np.ndarray
 type VectArray = Iterable[Vect] | np.ndarray
@@ -17,15 +17,19 @@ type AlphaArray = Iterable[float] | np.ndarray
 type Rgba = Iterable[float] | np.ndarray
 type RgbaArray = Iterable[Rgba] | np.ndarray
 
+type RangeSpecifier = tuple[float, float] | tuple[float, float, float]
 
-@runtime_checkable
-class SupportsInterpolate[T](Protocol):
-    @classmethod
-    def align_for_interpolate(cls, obj1: object, obj2: object) -> AlignedData[T]: ...
-
-    def interpolate(self, obj1: object, obj2: object, *, path_func): ...
+type ForeverType = types.EllipsisType
 
 
 @runtime_checkable
 class SupportsApartAlpha(Protocol):
     def apart_alpha(self, n: int) -> None: ...
+
+
+class SupportsAnim(Protocol):
+    '''
+    有些直接是 :class:`~.Animation`，而有些不是，但可以转化为 :class:`~.Animation，
+    因此用 :class:`SupportsAnim` 作为它们的统称，并会被 `AnimGroup._get_animation_objects` 统一转化
+    '''
+    def __anim__(self) -> 'Animation': ...
